@@ -65,15 +65,14 @@ class RequestMessage:
     request_id: int = field(default_factory=lambda: get_next_int(10000, 32767))
 
     def encode_message(
-        self,
-        protocol: RoborockMessageProtocol,
-        security_data: SecurityData | None = None,
+        self, protocol: RoborockMessageProtocol, security_data: SecurityData | None = None, version: str = "1.0"
     ) -> RoborockMessage:
         """Convert the request message to a RoborockMessage."""
         return RoborockMessage(
             timestamp=self.timestamp,
             protocol=protocol,
             payload=self._as_payload(security_data=security_data),
+            version=version.encode(),
         )
 
     def _as_payload(self, security_data: SecurityData | None) -> bytes:
@@ -188,7 +187,7 @@ def create_map_response_decoder(security_data: SecurityData) -> Callable[[Roboro
         header, body = message.payload[:24], message.payload[24:]
         [endpoint, _, request_id, _] = struct.unpack("<8s8sH6s", header)
         if not endpoint.decode().startswith(security_data.endpoint):
-            _LOGGER.debug("Received map response requested not made by this device, ignoring.")
+            _LOGGER.debug("Received map response not requested by this device, ignoring.")
             return None
         try:
             decrypted = Utils.decrypt_cbc(body, security_data.nonce)
