@@ -3,32 +3,27 @@ import logging
 from asyncio import Lock, TimerHandle, Transport, get_running_loop
 from collections.abc import Callable
 from dataclasses import dataclass
+from enum import StrEnum
 
 import async_timeout
 
 from .. import CommandVacuumError, DeviceData, RoborockCommand
 from ..api import RoborockClient
 from ..exceptions import RoborockConnectionException, RoborockException, VacuumError
-from ..protocol import Decoder, Encoder, create_local_decoder, create_local_encoder
-from ..protocols.v1_protocol import encode_local_payload
+from ..protocol import create_local_decoder, create_local_encoder
+from ..protocols.v1_protocol import RequestMessage
 from ..roborock_message import RoborockMessage, RoborockMessageProtocol
-from ..util import RoborockLoggerAdapter
+from ..util import RoborockLoggerAdapter, get_next_int
 from .roborock_client_v1 import CLOUD_REQUIRED, RoborockClientV1
 
 _LOGGER = logging.getLogger(__name__)
 
 
-_HELLO_REQUEST_MESSAGE = RoborockMessage(
-    protocol=RoborockMessageProtocol.HELLO_REQUEST,
-    seq=1,
-    random=22,
-)
+class LocalProtocolVersion(StrEnum):
+    """Supported local protocol versions. Different from vacuum protocol versions."""
 
-_PING_REQUEST_MESSAGE = RoborockMessage(
-    protocol=RoborockMessageProtocol.PING_REQUEST,
-    seq=2,
-    random=23,
-)
+    L01 = "L01"
+    V1 = "1.0"
 
 
 @dataclass
@@ -50,7 +45,12 @@ class _LocalProtocol(asyncio.Protocol):
 class RoborockLocalClientV1(RoborockClientV1, RoborockClient):
     """Roborock local client for v1 devices."""
 
-    def __init__(self, device_data: DeviceData, queue_timeout: int = 4):
+    def __init__(
+        self,
+        device_data: DeviceData,
+        queue_timeout: int = 4,
+        local_protocol_version: LocalProtocolVersion | None = None,
+    ):
         """Initialize the Roborock local client."""
         if device_data.host is None:
             raise RoborockException("Host is required")
@@ -60,14 +60,20 @@ class RoborockLocalClientV1(RoborockClientV1, RoborockClient):
         self.transport: Transport | None = None
         self._mutex = Lock()
         self.keep_alive_task: TimerHandle | None = None
-        RoborockClientV1.__init__(self, device_data, "abc")
+        RoborockClientV1.__init__(self, device_data, security_data=None)
         RoborockClient.__init__(self, device_data)
         self._local_protocol = _LocalProtocol(self._data_received, self._connection_lost)
-        self._encoder: Encoder = create_local_encoder(device_data.device.local_key)
-        self._decoder: Decoder = create_local_decoder(device_data.device.local_key)
+        self._local_protocol_version = local_protocol_version
+        self._connect_nonce = get_next_int(10000, 32767)
+        self._ack_nonce: int | None = None
+        self._set_encoder_decoder()
         self.queue_timeout = queue_timeout
         self._logger = RoborockLoggerAdapter(device_data.device.name, _LOGGER)
 
+    @property
+    def local_protocol_version(self) -> LocalProtocolVersion:
+        return LocalProtocolVersion.V1 if self._local_protocol_version is None else self._local_protocol_version
+
     def _data_received(self, message):
         """Called when data is received from the transport."""
         parsed_msg = self._decoder(message)
@@ -121,14 +127,77 @@ class RoborockLocalClientV1(RoborockClientV1, RoborockClient):
         async with self._mutex:
             self._sync_disconnect()
 
-    async def hello(self):
+    def _set_encoder_decoder(self):
+        """Updates the encoder decoder. These are updated with nonces after the first hello.
+        Only L01 uses the nonces."""
+        self._encoder = create_local_encoder(self.device_info.device.local_key, self._connect_nonce, self._ack_nonce)
+        self._decoder = create_local_decoder(self.device_info.device.local_key, self._connect_nonce, self._ack_nonce)
+
+    async def _do_hello(self, local_protocol_version: LocalProtocolVersion) -> bool:
+        """Perform the initial handshaking."""
+        self._logger.debug(
+            "Attempting to use the %s protocol for client %s...",
+            local_protocol_version,
+            self.device_info.device.duid,
+        )
+        request = RoborockMessage(
+            protocol=RoborockMessageProtocol.HELLO_REQUEST,
+            version=local_protocol_version.encode(),
+            random=self._connect_nonce,
+            seq=1,
+        )
         try:
-            return await self._send_message(_HELLO_REQUEST_MESSAGE)
-        except Exception as e:
-            self._logger.error(e)
+            response = await self._send_message(
+                roborock_message=request,
+                request_id=request.seq,
+                response_protocol=RoborockMessageProtocol.HELLO_RESPONSE,
+            )
+            self._ack_nonce = response.random
+            self._set_encoder_decoder()
+            self._local_protocol_version = local_protocol_version
+
+            self._logger.debug(
+                "Client %s speaks the %s protocol.",
+                self.device_info.device.duid,
+                local_protocol_version,
+            )
+            return True
+        except RoborockException as e:
+            self._logger.debug(
+                "Client %s did not respond or does not speak the %s protocol. %s",
+                self.device_info.device.duid,
+                local_protocol_version,
+                e,
+            )
+            return False
+
+    async def hello(self):
+        """Send hello to the device to negotiate protocol."""
+        if self._local_protocol_version:
+            # version is forced
+            if not await self._do_hello(self._local_protocol_version):
+                raise RoborockException(f"Failed to connect to device with protocol {self._local_protocol_version}")
+        else:
+            # try 1.0, then L01
+            if not await self._do_hello(LocalProtocolVersion.V1):
+                if not await self._do_hello(LocalProtocolVersion.L01):
+                    raise RoborockException("Failed to connect to device with any known protocol")
 
     async def ping(self) -> None:
-        await self._send_message(_PING_REQUEST_MESSAGE)
+        ping_message = RoborockMessage(
+            protocol=RoborockMessageProtocol.PING_REQUEST, version=self.local_protocol_version.encode()
+        )
+        await self._send_message(
+            roborock_message=ping_message,
+            request_id=ping_message.seq,
+            response_protocol=RoborockMessageProtocol.PING_RESPONSE,
+        )
+
+    async def _validate_connection(self) -> None:
+        if not self.should_keepalive():
+            self._logger.info("Resetting Roborock connection due to keepalive timeout")
+            await self.async_disconnect()
+        await self.async_connect()
 
     def _send_msg_raw(self, data: bytes):
         try:
@@ -145,27 +214,29 @@ class RoborockLocalClientV1(RoborockClientV1, RoborockClient):
     ):
         if method in CLOUD_REQUIRED:
             raise RoborockException(f"Method {method} is not supported over local connection")
-
-        roborock_message = encode_local_payload(method, params)
-        self._logger.debug("Building message id %s for method %s", roborock_message.get_request_id(), method)
-        return await self._send_message(roborock_message, method, params)
+        request_message = RequestMessage(method=method, params=params)
+        roborock_message = request_message.encode_message(
+            RoborockMessageProtocol.GENERAL_REQUEST,
+            version=self.local_protocol_version,
+        )
+        self._logger.debug("Building message id %s for method %s", request_message.request_id, method)
+        return await self._send_message(
+            roborock_message,
+            request_id=request_message.request_id,
+            response_protocol=RoborockMessageProtocol.GENERAL_REQUEST,
+            method=method,
+            params=params,
+        )
 
     async def _send_message(
         self,
         roborock_message: RoborockMessage,
+        request_id: int,
+        response_protocol: int,
         method: str | None = None,
         params: list | dict | int | None = None,
     ) -> RoborockMessage:
-        await self.validate_connection()
-        request_id: int | None
-        if not method or not method.startswith("get"):
-            request_id = roborock_message.seq
-            response_protocol = request_id + 1
-        else:
-            request_id = roborock_message.get_request_id()
-            response_protocol = RoborockMessageProtocol.GENERAL_REQUEST
-        if request_id is None:
-            raise RoborockException(f"Failed build message {roborock_message}")
+        await self._validate_connection()
         msg = self._encoder(roborock_message)
         if method:
             self._logger.debug(f"id={request_id} Requesting method {method} with {params}")
